@@ -5,7 +5,7 @@
 // Author:      Aleksandras Gluchovas <mailto:alex@soften.ktu.lt>
 // Modified by:
 // Created:     06/09/98
-// RCS-ID:      $Id: controlbar.h,v 1.19 2005/03/21 22:52:59 VZ Exp $
+// RCS-ID:      $Id: controlbar.h,v 1.21 2005/05/23 10:11:55 ABX Exp $
 // Copyright:   (c) Aleksandras Gluchovas
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -904,10 +904,10 @@ public:
     bool          mHasLeftHandle;
     bool          mHasRightHandle;
 
-    // determines if this bar can float. The layout's setting as priority. For 
+    // determines if this bar can float. The layout's setting as priority. For
     // example, if the layout's mFloatingOn is false, this setting is irrelevant
     // since nothing will float at all. If the layout's floating is on, use this
-    // setting to prevent specific bars from floating. In other words, all bars 
+    // setting to prevent specific bars from floating. In other words, all bars
     // float by default and floating can be turned off on individual bars.
     bool          mFloatingOn;    // default: ON (which is also the layout's mFloatingOn default setting)
 
@@ -1002,7 +1002,7 @@ which are usually the same for all panes in
 a frame layout.
 */
 
-class cbCommonPaneProperties : public wxObject
+class WXDLLIMPEXP_FL cbCommonPaneProperties : public wxObject
 {
     DECLARE_DYNAMIC_CLASS( cbCommonPaneProperties )
 
@@ -1036,11 +1036,11 @@ class cbCommonPaneProperties : public wxObject
     cbCommonPaneProperties(void);
 
         // Copy constructor
-    
+
     cbCommonPaneProperties(const cbCommonPaneProperties&);
 
         // Assignment operator
-    
+
     cbCommonPaneProperties& operator=(const cbCommonPaneProperties&);
 };
 
@@ -1461,7 +1461,7 @@ areas of frame layout that actually need to be updated. This should be extended
 to implement a custom updating strategy.
 */
 
-class cbUpdatesManagerBase : public wxObject
+class WXDLLIMPEXP_FL cbUpdatesManagerBase : public wxObject
 {
     DECLARE_ABSTRACT_CLASS( cbUpdatesManagerBase )
 
