@@ -3,7 +3,7 @@
 // Purpose:     XML resource handler for radio box
 // Author:      Bob Mitchell
 // Created:     2000/03/21
-// RCS-ID:      $Id: xh_radbx.h,v 1.4 2003/07/04 13:33:41 VS Exp $
+// RCS-ID:      $Id: xh_radbx.h,v 1.5 2005/01/07 21:27:19 VS Exp $
 // Copyright:   (c) 2000 Bob Mitchell and Verant Interactive
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -11,7 +11,7 @@
 #ifndef _WX_XH_RADBX_H_
 #define _WX_XH_RADBX_H_
 
-#if defined(__GNUG__) && !defined(__APPLE__)
+#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
 #pragma interface "xh_radbx.h"
 #endif
 
