@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: combobox.cpp,v 1.57 2005/04/03 22:57:32 RD Exp $
+// RCS-ID:      $Id: combobox.cpp,v 1.60 2005/05/10 06:29:21 SC Exp $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -22,9 +22,7 @@
 #include "wx/menu.h"
 #include "wx/mac/uma.h"
 
-#if !USE_SHARED_LIBRARY
 IMPLEMENT_DYNAMIC_CLASS(wxComboBox, wxControl)
-#endif
 
 // composite combobox implementation by Dan "Bud" Keith bud@otsys.com
 
@@ -84,6 +82,14 @@ protected:
             if (m_cb->GetParent()->GetEventHandler()->ProcessEvent(NavEvent))
                     return;
         }
+
+        // send the event to the combobox class in case the user has bound EVT_CHAR
+        wxKeyEvent kevt(event);
+        kevt.SetEventObject(m_cb);
+        if (m_cb->GetEventHandler()->ProcessEvent(kevt))
+            // If the event was handled and not skipped then we're done
+            return;
+        
         if ( event.GetKeyCode() == WXK_RETURN )
         {
             wxCommandEvent event(wxEVT_COMMAND_TEXT_ENTER, m_cb->GetId());
@@ -119,18 +125,30 @@ protected:
         event.Skip();
     }
 
-    // Use the KeyUp as a naive approximation for TEXT_UPDATED, even though it is somewhat delayed
-    // but this is less complicated than dealing with idle-ness, and is much better than nothing
     void OnKeyUp( wxKeyEvent& event )
     {
-        if ( event.GetKeyCode() != WXK_RETURN && event.GetKeyCode() != WXK_TAB )
-        {
-            wxCommandEvent event(wxEVT_COMMAND_TEXT_UPDATED, m_cb->GetId());
-            event.SetString( GetValue() );
-            event.SetEventObject( m_cb );
-            m_cb->GetEventHandler()->ProcessEvent(event);
-        }
+        event.SetEventObject(m_cb);
+        event.SetId(m_cb->GetId());
+        if (! m_cb->GetEventHandler()->ProcessEvent(event))
+            event.Skip();
     }
+
+    void OnKeyDown( wxKeyEvent& event )
+    {
+        event.SetEventObject(m_cb);
+        event.SetId(m_cb->GetId());
+        if (! m_cb->GetEventHandler()->ProcessEvent(event))
+            event.Skip();
+    }
+    
+    void OnText( wxCommandEvent& event )
+    {
+        event.SetEventObject(m_cb);
+        event.SetId(m_cb->GetId());
+        if (! m_cb->GetEventHandler()->ProcessEvent(event))
+            event.Skip();
+    }        
+
 private:
     wxComboBox *m_cb;
 
@@ -138,15 +156,17 @@ private:
 };
 
 BEGIN_EVENT_TABLE(wxComboBoxText, wxTextCtrl)
-    EVT_CHAR( wxComboBoxText::OnChar)
-    EVT_KEY_UP( wxComboBoxText::OnKeyUp)
+    EVT_KEY_DOWN( wxComboBoxText::OnKeyDown)
+    EVT_CHAR(     wxComboBoxText::OnChar)
+    EVT_KEY_UP(   wxComboBoxText::OnKeyUp)
+    EVT_TEXT( -1, wxComboBoxText::OnText)
 END_EVENT_TABLE()
 
 class wxComboBoxChoice : public wxChoice
 {
 public:
     wxComboBoxChoice(wxComboBox *cb, int style)
-        : wxChoice( cb , 1 )
+        : wxChoice( cb , 1 , wxDefaultPosition , wxDefaultSize , 0 , NULL , style & (wxCB_SORT) )
     {
         m_cb = cb;
     }
