@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: dccg.cpp,v 1.19 2005/03/25 17:40:55 SC Exp $
+// RCS-ID:      $Id: dccg.cpp,v 1.21 2005/05/10 06:28:21 SC Exp $
 // Copyright:   (c) Stefan Csomor
 // Licence:       wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -40,9 +40,7 @@ using namespace std ;
 #include <FixMath.h>
 #include <CGContext.h>
 
-#if !USE_SHARED_LIBRARY
 IMPLEMENT_ABSTRACT_CLASS(wxDC, wxObject)
-#endif
 
 //-----------------------------------------------------------------------------
 // constants
@@ -1603,7 +1601,7 @@ void  wxDC::Clear(void)
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
                     if ( HIThemeSetFill != 0 )
                     {
-                        HIThemeSetFill( m_backgroundBrush.MacGetTheme() , cg ) ;
+                        HIThemeSetFill( m_backgroundBrush.MacGetTheme() , NULL , cg , kHIThemeOrientationNormal ) ;
                         CGContextFillRect(cg, rect);
 
                     }
