@@ -4,12 +4,12 @@
 // Author:      Julian Smart
 // Modified by: 
 // Created:     04/01/98
-// RCS-ID:      $Id: pnghand.cpp,v 1.20 2004/09/22 20:47:10 RN Exp $
+// RCS-ID:      $Id: pnghand.cpp,v 1.23 2005/02/12 11:34:04 SC Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#ifdef __GNUG__
+#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
 #  pragma implementation "pngread.h"
 #  pragma implementation "pnghand.h"
 #endif
@@ -21,12 +21,23 @@
 #  pragma hdrstop
 #endif
 
-#if wxUSE_LIBPNG
+#if 0 // wxUSE_LIBPNG
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#if defined(__DARWIN__)
+/* MW's math routines do certain things if __FP__ (the include guard around
+CarbonCore's fp.h) is defined.  CarbonCore's fp.h does certain things if
+__cmath__ is defined so it seems the easy thing to do is to make sure
+__cmath__ is effectively not defined which counteracts the MWERKS check
+then when the real cmath is included everything will be okay.
+*/
+#include <CoreServices/CoreServices.h>
+//#include <cmath>
+#endif //defined(__DARWIN__)
+
 #if wxUSE_IOSTREAMH
 #   include <fstream.h>
 #else
