@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: bitmap.cpp,v 1.127 2004/10/06 20:18:41 ABX Exp $
+// RCS-ID:      $Id: bitmap.cpp,v 1.130 2005/05/25 17:06:47 RD Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -310,11 +310,38 @@ bool wxBitmap::CopyFromIconOrCursor(const wxGDIImage& icon)
 
     refData->m_hBitmap = (WXHBITMAP)iconInfo.hbmColor;
 
-    // the mask returned by GetIconInfo() is inversed compared to the usual
-    // wxWin convention
-    refData->SetMask(wxInvertMask(iconInfo.hbmMask, w, h));
-
-
+#if wxUSE_WXDIB
+    // If the icon is 32 bits per pixel then it may have alpha channel data,
+    // although there are some icons that are 32 bpp but have no alpha... So
+    // convert to a DIB and manually check the 4th byte for each pixel.
+    BITMAP bm;
+    if ( ::GetObject(iconInfo.hbmColor, sizeof(BITMAP), (LPVOID)&bm)
+         && bm.bmBitsPixel == 32)
+    {
+        wxDIB dib(iconInfo.hbmColor);
+        if (dib.IsOk())
+        {
+            unsigned char* pixels = dib.GetData();
+            for (int idx=0; idx<w*h*4; idx+=4)
+            {
+                if (pixels[idx+3] != 0) 
+                {
+                    // If there is an alpha byte that is non-zero then set the
+                    // alpha flag and bail out of the loop.
+                    refData->m_hasAlpha = true;
+                    break;
+                }
+            }
+        }
+    }
+#endif
+    if ( !refData->m_hasAlpha )
+    {
+        // the mask returned by GetIconInfo() is inversed compared to the usual
+        // wxWin convention
+        refData->SetMask(wxInvertMask(iconInfo.hbmMask, w, h));
+    }
+    
     // delete the old one now as we don't need it any more
     ::DeleteObject(iconInfo.hbmMask);
 
@@ -843,9 +870,42 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth, WXHDC hdc)
     // finally also set the mask if we have one
     if ( image.HasMask() )
     {
-        SetMask(new wxMask(*this, wxColour(image.GetMaskRed(),
-                                           image.GetMaskGreen(),
-                                           image.GetMaskBlue())));
+        const size_t len  = 2*((w+15)/16);
+        BYTE *src  = image.GetData();
+        BYTE *data = new BYTE[h*len];
+        memset(data, 0, h*len);
+        BYTE r = image.GetMaskRed(),
+             g = image.GetMaskGreen(),
+             b = image.GetMaskBlue();
+        BYTE *dst = data;
+        for ( int y = 0; y < h; y++, dst += len )
+        {
+            BYTE *dstLine = dst;
+            BYTE mask = 0x80;
+            for ( int x = 0; x < w; x++, src += 3 )
+            {
+                if (src[0] != r || src[1] != g || src[2] != b)
+                    *dstLine |= mask;
+
+                if ( (mask >>= 1) == 0 )
+                {
+                    dstLine++;
+                    mask = 0x80;
+                }
+            }
+        }
+
+        hbitmap = ::CreateBitmap(w, h, 1, 1, data);
+        if ( !hbitmap )
+        {
+            wxLogLastError(_T("CreateBitmap(mask)"));
+        }
+        else
+        {
+            SetMask(new wxMask((WXHBITMAP)hbitmap));
+        }
+
+        delete data;
     }
 
     return true;
@@ -1092,6 +1152,15 @@ wxMask *wxBitmap::GetMask() const
     return GetBitmapData() ? GetBitmapData()->GetMask() : (wxMask *) NULL;
 }
 
+wxBitmap wxBitmap::GetMaskBitmap() const
+{
+    wxBitmap bmp;
+    wxMask *mask = GetMask();
+    if ( mask )
+        bmp.SetHBITMAP(mask->GetMaskBitmap());
+    return bmp;
+}
+
 #ifdef __WXDEBUG__
 
 wxDC *wxBitmap::GetSelectedInto() const
