@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.03.03 (replaces the old file with the same name)
-// RCS-ID:      $Id: dib.cpp,v 1.58 2004/10/07 13:36:40 ABX Exp $
+// RCS-ID:      $Id: dib.cpp,v 1.59 2005/04/23 18:59:30 JS Exp $
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
 // License:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -48,10 +48,6 @@
     #include <memory.h>
 #endif
 
-#ifdef __GNUWIN32_OLD__
-    #include "wx/msw/gnuwin32/extra.h"
-#endif
-
 #include "wx/image.h"
 #include "wx/msw/dib.h"
 
