@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: textctrl.cpp,v 1.228 2004/10/08 18:49:07 ABX Exp $
+// RCS-ID:      $Id: textctrl.cpp,v 1.246 2005/05/18 02:26:37 RD Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -39,6 +39,7 @@
     #include "wx/log.h"
     #include "wx/app.h"
     #include "wx/menu.h"
+    #include "wx/math.h"
 #endif
 
 #include "wx/module.h"
@@ -83,25 +84,63 @@
 class wxRichEditModule : public wxModule
 {
 public:
+    enum Version
+    {
+        Version_1,          // riched32.dll
+        Version_2or3,       // both use riched20.dll
+        Version_41,         // msftedit.dll (XP SP1 and Windows 2003)
+        Version_Max
+    };
+
     virtual bool OnInit();
     virtual void OnExit();
 
-    // load the richedit DLL of at least of required version
-    static bool Load(int version = 1);
+    // load the richedit DLL for the specified version of rich edit
+    static bool Load(Version version);
 
 private:
     // the handles to richedit 1.0 and 2.0 (or 3.0) DLLs
-    static HINSTANCE ms_hRichEdit[2];
+    static HINSTANCE ms_hRichEdit[Version_Max];
 
     DECLARE_DYNAMIC_CLASS(wxRichEditModule)
 };
 
-HINSTANCE wxRichEditModule::ms_hRichEdit[2] = { NULL, NULL };
+HINSTANCE wxRichEditModule::ms_hRichEdit[Version_Max] = { NULL, NULL, NULL };
 
 IMPLEMENT_DYNAMIC_CLASS(wxRichEditModule, wxModule)
 
 #endif // wxUSE_RICHEDIT
 
+// a small class used to set m_updatesCount to 0 (to filter duplicate events if
+// necessary) and to reset it back to -1 afterwards
+class UpdatesCountFilter
+{
+public:
+    UpdatesCountFilter(int& count)
+        : m_count(count)
+    {
+        wxASSERT_MSG( m_count == -1, _T("wrong initial m_updatesCount value") );
+
+        m_count = 0;
+    }
+
+    ~UpdatesCountFilter()
+    {
+        m_count = -1;
+    }
+
+    // return true if an event has been received
+    bool GotUpdate() const
+    {
+        return m_count == 1;
+    }
+
+private:
+    int& m_count;
+
+    DECLARE_NO_COPY_CLASS(UpdatesCountFilter)
+};
+
 // ----------------------------------------------------------------------------
 // event tables and other macros
 // ----------------------------------------------------------------------------
@@ -181,7 +220,7 @@ BEGIN_EVENT_TABLE(wxTextCtrl, wxControl)
     EVT_DROP_FILES(wxTextCtrl::OnDropFiles)
 
 #if wxUSE_RICHEDIT
-    EVT_RIGHT_UP(wxTextCtrl::OnRightClick)
+    EVT_CONTEXT_MENU(wxTextCtrl::OnContextMenu)
 #endif
 
     EVT_MENU(wxID_CUT, wxTextCtrl::OnCut)
@@ -218,17 +257,13 @@ void wxTextCtrl::Init()
 #endif // wxUSE_RICHEDIT
 
     m_privateContextMenu = NULL;
-    m_suppressNextUpdate = false;
+    m_updatesCount = -1;
     m_isNativeCaretShown = true;
 }
 
 wxTextCtrl::~wxTextCtrl()
 {
-    if (m_privateContextMenu)
-    {
-        delete m_privateContextMenu;
-        m_privateContextMenu = NULL;
-    }
+    delete m_privateContextMenu;
 }
 
 bool wxTextCtrl::Create(wxWindow *parent, wxWindowID id,
@@ -254,6 +289,12 @@ bool wxTextCtrl::Create(wxWindow *parent, wxWindowID id,
     // do create the control - either an EDIT or RICHEDIT
     wxString windowClass = wxT("EDIT");
 
+#if defined(__POCKETPC__) || defined(__SMARTPHONE__)
+    // A control that capitalizes the first letter
+    if (style & wxTE_CAPITALIZE)
+        windowClass = wxT("CAPEDIT");
+#endif
+
 #if wxUSE_RICHEDIT
     if ( m_windowStyle & wxTE_AUTO_URL )
     {
@@ -270,63 +311,58 @@ bool wxTextCtrl::Create(wxWindow *parent, wxWindowID id,
     // we need to load the richedit DLL before creating the rich edit control
     if ( m_windowStyle & wxTE_RICH )
     {
-        static bool s_errorGiven = false;// MT-FIXME
-
-        // Which version do we need? Use 1.0 by default because it is much more
-        // like the the standard EDIT or 2.0 if explicitly requested, but use
-        // only 2.0 in Unicode mode as 1.0 doesn't support Unicode at all
+        // versions 2.0, 3.0 and 4.1 of rich edit are mostly compatible with
+        // each other but not with version 1.0, so we have separate flags for
+        // the version 1.0 and the others (and so m_verRichEdit may be 0 (plain
+        // EDIT control), 1 for version 1.0 or 2 for any higher version)
         //
-        // TODO: RichEdit 3.0 is apparently capable of emulating RichEdit 1.0
-        //       (and thus EDIT) much better than RichEdit 2.0 so we probably
-        //       should use 3.0 if available as it is the best of both worlds -
-        //       but as I can't test it right now I don't do it (VZ)
+        // notice that 1.0 has no Unicode support at all so in Unicode build we
+        // must use another version
+
 #if wxUSE_UNICODE
-        const int verRichEdit = 2;
+        m_verRichEdit = 2;
 #else // !wxUSE_UNICODE
-        int verRichEdit = m_windowStyle & wxTE_RICH2 ? 2 : 1;
+        m_verRichEdit = m_windowStyle & wxTE_RICH2 ? 2 : 1;
 #endif // wxUSE_UNICODE/!wxUSE_UNICODE
 
-        // only give the error msg once if the DLL can't be loaded
-        if ( !s_errorGiven )
+        if ( m_verRichEdit == 2 )
         {
-            // try to load the RichEdit DLL (will do nothing if already done)
-            if ( !wxRichEditModule::Load(verRichEdit) )
+            if ( wxRichEditModule::Load(wxRichEditModule::Version_41) )
             {
-#if !wxUSE_UNICODE
-                // try another version?
-                verRichEdit = 3 - verRichEdit; // 1 <-> 2
-
-                if ( !wxRichEditModule::Load(verRichEdit) )
-#endif // wxUSE_UNICODE
-                {
-                    wxLogError(_("Impossible to create a rich edit control, using simple text control instead. Please reinstall riched32.dll"));
-
-                    s_errorGiven = true;
-                }
+                // yes, class name for version 4.1 really is 5.0
+                windowClass = _T("RICHEDIT50W");
+            }
+            else if ( wxRichEditModule::Load(wxRichEditModule::Version_2or3) )
+            {
+                windowClass = _T("RichEdit20")
+#if wxUSE_UNICODE
+                              _T("W");
+#else // ANSI
+                              _T("A");
+#endif // Unicode/ANSI
+            }
+            else // failed to load msftedit.dll and riched20.dll
+            {
+                m_verRichEdit = 1;
             }
         }
 
-        // have we managed to load any richedit version?
-        if ( !s_errorGiven )
+        if ( m_verRichEdit == 1 )
         {
-            m_verRichEdit = verRichEdit;
-            if ( m_verRichEdit == 1 )
+            if ( wxRichEditModule::Load(wxRichEditModule::Version_1) )
             {
-                windowClass = wxT("RICHEDIT");
+                windowClass = _T("RICHEDIT");
             }
-            else
+            else // failed to load any richedit control DLL
             {
-#ifndef RICHEDIT_CLASS
-                wxString RICHEDIT_CLASS;
-                RICHEDIT_CLASS.Printf(_T("RichEdit%d0"), m_verRichEdit);
-#if wxUSE_UNICODE
-                RICHEDIT_CLASS += _T('W');
-#else // ANSI
-                RICHEDIT_CLASS += _T('A');
-#endif // Unicode/ANSI
-#endif // !RICHEDIT_CLASS
+                // only give the error msg once if the DLL can't be loaded
+                static bool s_errorGiven = false; // MT ok as only used by GUI
 
-                windowClass = RICHEDIT_CLASS;
+                wxLogError(_("Impossible to create a rich edit control, using simple text control instead. Please reinstall riched32.dll"));
+
+                s_errorGiven = true;
+
+                m_verRichEdit = 0;
             }
         }
     }
@@ -346,8 +382,6 @@ bool wxTextCtrl::Create(wxWindow *parent, wxWindowID id,
     if ( !MSWCreateControl(windowClass, msStyle, pos, size, valueWin) )
         return false;
 
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
-
 #if wxUSE_RICHEDIT
     if ( IsRich() )
     {
@@ -360,6 +394,17 @@ bool wxTextCtrl::Create(wxWindow *parent, wxWindowID id,
             // we also need EN_MSGFILTER for richedit 1.0 for the reasons
             // explained in its handler
            mask |= ENM_MOUSEEVENTS;
+
+           // we also need to force the appearance of the vertical scrollbar
+           // initially as otherwise the control doesn't refresh correctly
+           // after resize: but once the vertical scrollbar had been shown
+           // (even if it's subsequently hidden) it does
+           //
+           // this is clearly a bug and for now it has been only noticed under
+           // Windows XP, so if we're sure it works correctly under other
+           // systems we could do this only for XP
+           SetSize(-1, 1); // 1 is small enough to force vert scrollbar
+           SetSize(size);
         }
         else if ( m_windowStyle & wxTE_AUTO_URL )
         {
@@ -756,13 +801,8 @@ wxTextCtrl::StreamIn(const wxString& value,
     // the cast below is needed for broken (very) old mingw32 headers
     eds.pfnCallback = (EDITSTREAMCALLBACK)wxRichEditStreamIn;
 
-    // we're going to receive 2 EN_CHANGE notifications if we got any selection
-    // (same problem as in DoWriteText())
-    if ( selectionOnly && HasSelection() )
-    {
-        // so suppress one of them
-        m_suppressNextUpdate = true;
-    }
+    // same problem as in DoWriteText(): we can get multiple events here
+    UpdatesCountFilter ucf(m_updatesCount);
 
     ::SendMessage(GetHwnd(), EM_STREAMIN,
                   SF_TEXT |
@@ -770,6 +810,8 @@ wxTextCtrl::StreamIn(const wxString& value,
                   (selectionOnly ? SFF_SELECTION : 0),
                   (LPARAM)&eds);
 
+    wxASSERT_MSG( ucf.GotUpdate(), _T("EM_STREAMIN didn't send EN_UPDATE?") );
+
     if ( eds.dwError )
     {
         wxLogLastError(_T("EM_STREAMIN"));
@@ -911,40 +953,22 @@ void wxTextCtrl::DoWriteText(const wxString& value, bool selectionOnly)
 #endif // wxUSE_RICHEDIT
     {
         // in some cases we get 2 EN_CHANGE notifications after the SendMessage
-        // call below which is confusing for the client code and so should be
-        // avoided
-        //
-        // these cases are: (a) plain EDIT controls if EM_REPLACESEL is used
-        // and there is a non empty selection currently and (b) rich text
-        // controls in any case
-        if (
-#if wxUSE_RICHEDIT
-            IsRich() ||
-#endif // wxUSE_RICHEDIT
-            (selectionOnly && HasSelection()) )
-        {
-            m_suppressNextUpdate = true;
-        }
+        // call (this happens for plain EDITs with EM_REPLACESEL and under some
+        // -- undetermined -- conditions with rich edit) and sometimes we don't
+        // get any events at all (plain EDIT with WM_SETTEXT), so ensure that
+        // we generate exactly one of them by ignoring all but the first one in
+        // SendUpdateEvent() and generating one ourselves if we hadn't got any
+        // notifications from Windows
+        UpdatesCountFilter ucf(m_updatesCount);
 
         ::SendMessage(GetHwnd(), selectionOnly ? EM_REPLACESEL : WM_SETTEXT,
                       0, (LPARAM)valueDos.c_str());
 
-        // OTOH, non rich text controls don't generate any events at all when
-        // we use WM_SETTEXT -- have to emulate them here
-        if ( !selectionOnly
-#if wxUSE_RICHEDIT
-                && !IsRich()
-#endif // wxUSE_RICHEDIT
-           )
+        if ( !ucf.GotUpdate() )
         {
-            // Windows already sends an update event for single-line
-            // controls.
-            if ( m_windowStyle & wxTE_MULTILINE )
-                SendUpdateEvent();
+            SendUpdateEvent();
         }
     }
-
-    AdjustSpaceLimit();
 }
 
 void wxTextCtrl::AppendText(const wxString& text)
@@ -954,7 +978,8 @@ void wxTextCtrl::AppendText(const wxString& text)
     WriteText(text);
 
 #if wxUSE_RICHEDIT
-    if ( IsMultiLine() && GetRichVersion() > 1 )
+    // don't do this if we're frozen, saves some time
+    if ( !IsFrozen() && IsMultiLine() && GetRichVersion() > 1 )
     {
         // setting the caret to the end and showing it simply doesn't work for
         // RichEdit 2.0 -- force it to still do what we want
@@ -1094,7 +1119,9 @@ void wxTextCtrl::SetInsertionPointEnd()
     // doing it results in horrible flicker when appending big amounts of text
     // to the control in a few chunks (see DoAddText() test in the text sample)
     if ( GetInsertionPoint() == GetLastPosition() )
+    {
         return;
+    }
 
     long pos;
 
@@ -1130,7 +1157,7 @@ long wxTextCtrl::GetInsertionPoint() const
     return Pos & 0xFFFF;
 }
 
-long wxTextCtrl::GetLastPosition() const
+wxTextPos wxTextCtrl::GetLastPosition() const
 {
     int numLines = GetNumberOfLines();
     long posStartLastLine = XYToPosition(0, numLines - 1);
@@ -1212,7 +1239,7 @@ void wxTextCtrl::DoSetSelection(long from, long to, bool scrollCaret)
         ::SendMessage(hWnd, EM_SETSEL, (WPARAM)from, (LPARAM)to);
     }
 
-    if ( scrollCaret )
+    if ( scrollCaret && !IsFrozen() )
     {
 #if wxUSE_RICHEDIT
         // richedit 3.0 (i.e. the version living in riched20.dll distributed
@@ -1793,18 +1820,29 @@ WXLRESULT wxTextCtrl::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPara
 
 bool wxTextCtrl::SendUpdateEvent()
 {
-    // is event reporting suspended?
-    if ( m_suppressNextUpdate )
+    switch ( m_updatesCount )
     {
-        // do process the next one
-        m_suppressNextUpdate = false;
+        case 0:
+            // remember that we've got an update
+            m_updatesCount++;
+            break;
 
-        return false;
+        case 1:
+            // we had already sent one event since the last control modification
+            return false;
+
+        default:
+            wxFAIL_MSG( _T("unexpected wxTextCtrl::m_updatesCount value") );
+            // fall through
+
+        case -1:
+            // we hadn't updated the control ourselves, this event comes from
+            // the user, don't need to ignore it nor update the count
+            break;
     }
 
     wxCommandEvent event(wxEVT_COMMAND_TEXT_UPDATED, GetId());
     InitCommandEvent(event);
-    event.SetString(GetValue());
 
     return ProcessCommand(event);
 }
@@ -1848,38 +1886,12 @@ bool wxTextCtrl::MSWCommand(WXUINT param, WXWORD WXUNUSED(id))
     return true;
 }
 
-WXHBRUSH wxTextCtrl::OnCtlColor(WXHDC pDC, WXHWND WXUNUSED(pWnd), WXUINT WXUNUSED(nCtlColor),
-#if wxUSE_CTL3D
-                               WXUINT message,
-                               WXWPARAM wParam,
-                               WXLPARAM lParam
-#else
-                               WXUINT WXUNUSED(message),
-                               WXWPARAM WXUNUSED(wParam),
-                               WXLPARAM WXUNUSED(lParam)
-#endif
-    )
+WXHBRUSH wxTextCtrl::MSWControlColor(WXHDC hDC, WXHWND hWnd)
 {
-#if wxUSE_CTL3D
-    if ( m_useCtl3D )
-    {
-        HBRUSH hbrush = Ctl3dCtlColorEx(message, wParam, lParam);
-        return (WXHBRUSH) hbrush;
-    }
-#endif // wxUSE_CTL3D
+    if ( !IsEnabled() && !HasFlag(wxTE_MULTILINE) )
+        return MSWControlColorDisabled(hDC);
 
-    HDC hdc = (HDC)pDC;
-    wxColour colBack = GetBackgroundColour();
-
-    if (!IsEnabled() && (GetWindowStyle() & wxTE_MULTILINE) == 0)
-        colBack = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
-
-    ::SetBkColor(hdc, wxColourToRGB(colBack));
-    ::SetTextColor(hdc, wxColourToRGB(GetForegroundColour()));
-
-    wxBrush *brush = wxTheBrushList->FindOrCreateBrush(colBack, wxSOLID);
-
-    return (WXHBRUSH)brush->GetResourceHandle();
+    return wxTextCtrlBase::MSWControlColor(hDC, hWnd);
 }
 
 bool wxTextCtrl::AdjustSpaceLimit()
@@ -1948,7 +1960,7 @@ wxSize wxTextCtrl::DoGetBestSize() const
     int hText = cy;
     if ( m_windowStyle & wxTE_MULTILINE )
     {
-        hText *= wxMax(GetNumberOfLines(), 5);
+        hText *= wxMax(wxMin(GetNumberOfLines(), 10), 2); 
     }
     //else: for single line control everything is ok
 
@@ -2038,7 +2050,7 @@ void wxTextCtrl::OnUpdateSelectAll(wxUpdateUIEvent& event)
     event.Enable(GetLastPosition() > 0);
 }
 
-void wxTextCtrl::OnRightClick(wxMouseEvent& event)
+void wxTextCtrl::OnContextMenu(wxContextMenuEvent& event)
 {
 #if wxUSE_RICHEDIT
     if (IsRich())
@@ -2056,7 +2068,7 @@ void wxTextCtrl::OnRightClick(wxMouseEvent& event)
             m_privateContextMenu->AppendSeparator();
             m_privateContextMenu->Append(wxID_SELECTALL, _("Select &All"));
         }
-        PopupMenu(m_privateContextMenu, event.GetPosition());
+        PopupMenu(m_privateContextMenu);
         return;
     }
     else
@@ -2073,6 +2085,23 @@ void wxTextCtrl::OnSetFocus(wxFocusEvent& WXUNUSED(event))
     }
 }
 
+// ----------------------------------------------------------------------------
+// Default colors for MSW text control
+//
+// Set default background color to the native white instead of
+// the default wxSYS_COLOUR_BTNFACE (is triggered with wxNullColour).
+// ----------------------------------------------------------------------------
+
+wxVisualAttributes wxTextCtrl::GetDefaultAttributes() const
+{
+    wxVisualAttributes attrs;
+    attrs.font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+    attrs.colFg = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
+    attrs.colBg = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW); //white
+
+    return attrs;
+}
+
 // the rest of the file only deals with the rich edit controls
 #if wxUSE_RICHEDIT
 
@@ -2440,10 +2469,13 @@ bool wxTextCtrl::SetDefaultStyle(const wxTextAttr& style)
     if ( !wxTextCtrlBase::SetDefaultStyle(style) )
         return false;
 
-    // we have to do this or the style wouldn't apply for the text typed by the
-    // user
-    long posLast = GetLastPosition();
-    SetStyle(posLast, posLast, m_defaultStyle);
+    if ( IsEditable() )
+    {
+        // we have to do this or the style wouldn't apply for the text typed by
+        // the user
+        wxTextPos posLast = GetLastPosition();
+        SetStyle(posLast, posLast, m_defaultStyle);
+    }
 
     return true;
 }
@@ -2496,6 +2528,7 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
     (void) ::SendMessage(GetHwnd(), EM_GETCHARFORMAT,
                             SCF_SELECTION, (LPARAM)&cf) ;
 
+
     LOGFONT lf;
     lf.lfHeight = cf.yHeight;
     lf.lfWidth = 0;
@@ -2503,17 +2536,32 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
     lf.lfClipPrecision = 0;
     lf.lfEscapement = 0;
     wxStrcpy(lf.lfFaceName, cf.szFaceName);
+
+    //NOTE:  we _MUST_ set each of these values to _something_ since we
+    //do not call wxZeroMemory on the LOGFONT lf
     if (cf.dwEffects & CFE_ITALIC)
         lf.lfItalic = TRUE;
+    else
+        lf.lfItalic = FALSE;
+
     lf.lfOrientation = 0;
     lf.lfPitchAndFamily = cf.bPitchAndFamily;
     lf.lfQuality = 0;
+
     if (cf.dwEffects & CFE_STRIKEOUT)
         lf.lfStrikeOut = TRUE;
+    else
+        lf.lfStrikeOut = FALSE;
+
     if (cf.dwEffects & CFE_UNDERLINE)
         lf.lfUnderline = TRUE;
+    else
+        lf.lfUnderline = FALSE;
+
     if (cf.dwEffects & CFE_BOLD)
         lf.lfWeight = FW_BOLD;
+    else
+        lf.lfWeight = FW_NORMAL;
 
     wxFont font = wxCreateFontFromLogFont(& lf);
     if (font.Ok())
@@ -2585,6 +2633,8 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
 // wxRichEditModule
 // ----------------------------------------------------------------------------
 
+static const HINSTANCE INVALID_HINSTANCE = (HINSTANCE)-1;
+
 bool wxRichEditModule::OnInit()
 {
     // don't do anything - we will load it when needed
@@ -2595,7 +2645,7 @@ void wxRichEditModule::OnExit()
 {
     for ( size_t i = 0; i < WXSIZEOF(ms_hRichEdit); i++ )
     {
-        if ( ms_hRichEdit[i] )
+        if ( ms_hRichEdit[i] && ms_hRichEdit[i] != INVALID_HINSTANCE )
         {
             ::FreeLibrary(ms_hRichEdit[i]);
             ms_hRichEdit[i] = NULL;
@@ -2604,17 +2654,9 @@ void wxRichEditModule::OnExit()
 }
 
 /* static */
-bool wxRichEditModule::Load(int version)
+bool wxRichEditModule::Load(Version version)
 {
-    // we don't support loading richedit 3.0 as I don't know how to distinguish
-    // it from 2.0 anyhow
-    wxCHECK_MSG( version == 1 || version == 2, false,
-                 _T("incorrect richedit control version requested") );
-
-    // make it the index in the array
-    version--;
-
-    if ( ms_hRichEdit[version] == (HINSTANCE)-1 )
+    if ( ms_hRichEdit[version] == INVALID_HINSTANCE )
     {
         // we had already tried to load it and failed
         return false;
@@ -2626,16 +2668,21 @@ bool wxRichEditModule::Load(int version)
         return true;
     }
 
-    wxString dllname = version ? _T("riched20") : _T("riched32");
-    dllname += _T(".dll");
+    static const wxChar *dllnames[] =
+    {
+        _T("riched32"),
+        _T("riched20"),
+        _T("msftedit"),
+    };
 
-    ms_hRichEdit[version] = ::LoadLibrary(dllname);
+    wxCOMPILE_TIME_ASSERT( WXSIZEOF(dllnames) == Version_Max,
+                            RichEditDllNamesVersionsMismatch );
+
+    ms_hRichEdit[version] = ::LoadLibrary(dllnames[version]);
 
     if ( !ms_hRichEdit[version] )
     {
-        wxLogSysError(_("Could not load Rich Edit DLL '%s'"), dllname.c_str());
-
-        ms_hRichEdit[version] = (HINSTANCE)-1;
+        ms_hRichEdit[version] = INVALID_HINSTANCE;
 
         return false;
     }
