@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     24.09.01
-// RCS-ID:      $Id: toplevel.cpp,v 1.112 2005/04/02 21:42:42 JS Exp $
+// RCS-ID:      $Id: toplevel.cpp,v 1.114 2005/04/06 21:15:17 JS Exp $
 // Copyright:   (c) 2001 SciTech Software, Inc. (www.scitechsoft.com)
 // License:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -447,10 +447,6 @@ bool wxTopLevelWindowMSW::CreateFrame(const wxString& title,
 
     bool result = MSWCreate(wxCanvasClassName, title, pos, sz, flags, exflags);
 
-#ifdef __SMARTPHONE__
-    // Work around title non-display glitch
-    Show(false);
-#endif
     return result;
 }
 
@@ -548,9 +544,8 @@ bool wxTopLevelWindowMSW::Create(wxWindow *parent,
         );
     }
 
-    // Native look is full screen window on Smartphones and Standard SDK.
-    // TODO: check that we need this (if we're passing default values to ctor).
-    // Also check that there really is a difference between PocketPC and Smartphone in this regard.
+    // Note: if we include PocketPC in this test, dialogs can fail to show up,
+    // for example the text entry dialog in the dialogs sample. Problem with Maximise()?
 #if defined(__WXWINCE__) && (defined(__SMARTPHONE__) || defined(__WINCE_STANDARDSDK__))
     if ( style & wxMAXIMIZE )
     {
