@@ -5,7 +5,7 @@
 # Author:       Peter Yared
 #
 # Created:      8/15/03
-# CVS-ID:       $Id: PythonEditor.py,v 1.1 2005/04/08 22:51:50 RD Exp $
+# CVS-ID:       $Id: PythonEditor.py,v 1.2 2005/04/28 00:00:46 RD Exp $
 # Copyright:    (c) 2004-2005 ActiveGrid, Inc.
 # License:      wxWindows License
 #----------------------------------------------------------------------------
@@ -581,7 +581,6 @@ class PythonOptionsPanel(wx.Panel):
 # Icon Bitmaps - generated by encode_bitmaps.py
 #----------------------------------------------------------------------------
 from wx import ImageFromStream, BitmapFromImage
-from wx import EmptyIcon
 import cStringIO
 
 
@@ -609,6 +608,4 @@ def getPythonImage():
     return ImageFromStream(stream)
 
 def getPythonIcon():
-    icon = EmptyIcon()
-    icon.CopyFromBitmap(getPythonBitmap())
-    return icon
+    return wx.IconFromBitmap(getPythonBitmap())
